﻿using Performance.DtoModels;
using Performance.EntityModels;
using Performance.Infrastructure;
using Performance.Repository;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Performance.Services
{
    public class ModExtractService : IAutoInjection
    {
        private PerforModextractRepository perforModextractRepository;
        private PerforModitemRepository perforModitemRepository;
        private PerforModmoduleRepository perforModmoduleRepository;
        private PerforModspecialRepository perforModspecialRepository;
        private PerforHospitalconfigRepository perforHospitalconfigRepository;
        private PerforExtractRepository perforExtractRepository;

        public ModExtractService(PerforModextractRepository perforModextractRepository,
         PerforModitemRepository perforModitemRepository,
         PerforModmoduleRepository perforModmoduleRepository,
         PerforModspecialRepository perforModspecialRepository,
         PerforHospitalconfigRepository perforHospitalconfigRepository,
         PerforExtractRepository perforExtractRepository)
        {
            this.perforModextractRepository = perforModextractRepository;
            this.perforModitemRepository = perforModitemRepository;
            this.perforModmoduleRepository = perforModmoduleRepository;
            this.perforModspecialRepository = perforModspecialRepository;
            this.perforHospitalconfigRepository = perforHospitalconfigRepository;
            this.perforExtractRepository = perforExtractRepository;
        }

        /// <summary>
        /// 绩效数据抽取模板
        /// </summary>
        /// <returns></returns>
        public List<TitleValue> ExtractScheme(int hospitalId)
        {
            var titlevalue = new List<TitleValue>();
            var list = perforModextractRepository.GetEntities(t => t.HospitalId == hospitalId || t.HospitalId == null);
            if (list != null && list.Any())
                titlevalue = list.Select(t => new TitleValue { Title = t.EName, Value = t.Id.ToString() }).ToList();

            return titlevalue;
        }

        /// <summary>
        /// 费用类型
        /// </summary>
        /// <returns></returns>
        public List<TitleValue> FeeType()
        {
            var titlevalue = new List<TitleValue>();
            var type = EnumHelper.GetItems<SheetType>().Where(t => new List<int> { (int)SheetType.Income, (int)SheetType.Expend, (int)SheetType.SpecialUnit }.Contains(t.Value));
            if (type != null && type.Any())
            {
                titlevalue = type.Select(t => new TitleValue
                {
                    Title = t.Description,
                    Value = t.Value.ToString()
                }).ToList();
            }
            return titlevalue;
        }

        /// <summary>
        /// 绩效考核项费用来源
        /// </summary>
        /// <returns></returns>
        public ModFeeResponse FeeSource(ModModuleRequest request)
        {
            var configList = perforHospitalconfigRepository.GetEntities(t => t.HospitalId == request.HospitalId);
            if (configList != null && configList.Any())
            {
                var hospitalConfig = configList.First();
                var connection = ConnectionBuilder.Create(DatabaseType.SqlServer, hospitalConfig.DbSource, hospitalConfig.DbName, hospitalConfig.DbUser, hospitalConfig.DbPassword);
                int pagesize = 500, pagenum = 0;
                if (request.PageNum != 0)
                    pagenum = request.PageNum - 1;
                if (request.PageSize >= 500 && request.PageSize <= 10000)
                    pagesize = request.PageSize;

                string sql = $"select top {pagesize} t1.* from (select row_number() over(order by charge_name) as rownumber,* from (select min(code) code,charge_name from dic_fee where charge_name is not null and charge_name != '' group by charge_name)t )t1 where rownumber> {pagenum} * {pagesize};";
                var dataList = perforExtractRepository.ExecuteScript(connection, sql, null);
                if (dataList != null && dataList.Any())
                {
                    var list = new List<TitleValue>();
                    foreach (var num in dataList.Select(t => t.RowNumber).Distinct())
                    {
                        var data = new TitleValue
                        {
                            Title = dataList.First(t => t.RowNumber == num && t.ColumnName.ToLower() == "charge_name").Value.ToString(),
                            Value = dataList.First(t => t.RowNumber == num && t.ColumnName.ToLower() == "code").Value.ToString(),
                        };
                        list.Add(data);
                    }
                    sql = $"select count(*) num from (select min(code) code,charge_name from dic_fee where charge_name is not null and charge_name != '' group by charge_name)t ;";
                    connection = ConnectionBuilder.Create(DatabaseType.SqlServer, hospitalConfig.DbSource, hospitalConfig.DbName, hospitalConfig.DbUser, hospitalConfig.DbPassword);
                    dataList = perforExtractRepository.ExecuteScript(connection, sql, null);
                    return new ModFeeResponse
                    {
                        Total = (int)dataList.FirstOrDefault().Value,
                        PageNum = pagenum + 1,
                        PageSize = pagesize,
                        Data = list
                    };
                }
            }

            return new ModFeeResponse();
        }

        /// <summary>
        /// 费用字典新增
        /// </summary>
        /// <returns></returns>
        public mod_module AddModule(ModModuleRequest request)
        {
            mod_module entity = new mod_module
            {
                ModuleName = request.ModuleName,
                HospitalId = request.HospitalId,
                Description = request.Description,
                SheetType = request.SheetType,
                ExtractId = request.ExtractId
            };
            if (!perforModmoduleRepository.Add(entity))
                throw new PerformanceException("添加失败！");

            //收入  默认添加收费项
            if ((int)SheetType.Income == request.SheetType.Value)
            {
                var configList = perforHospitalconfigRepository.GetEntities(t => t.HospitalId == request.HospitalId);
                if (configList != null && configList.Any())
                {
                    var hospitalConfig = configList.First();
                    var connection = ConnectionBuilder.Create(DatabaseType.SqlServer, hospitalConfig.DbSource, hospitalConfig.DbName, hospitalConfig.DbUser, hospitalConfig.DbPassword);
                    string sql = "select distinct category from inpat_fee where category is not null union select distinct category from out_fee where category is not null;";
                    var dataList = perforExtractRepository.ExecuteScript(connection, sql, null);
                    if (dataList != null && dataList.Any())
                    {
                        var list = dataList.Select(t => new mod_item
                        {
                            ItemName = t.Value.ToString(),
                            FactorValue1 = 0m,
                            FactorValue2 = 0m,
                            ModuleId = entity.Id,
                            ExtractId = entity.ExtractId,
                        });
                        perforModitemRepository.AddRange(list.ToArray());
                    }
                }
            }
            return entity;
        }

        /// <summary>
        /// 费用字典
        /// </summary>
        /// <returns></returns>
        public List<mod_module> Module(int hospitalId)
        {
            var list = perforModmoduleRepository.GetEntities(t => t.HospitalId == hospitalId);
            return list;
        }

        /// <summary>
        /// 绩效模板修改
        /// </summary>
        /// <returns></returns>
        public mod_module EditModule(ModModuleRequest request)
        {
            var entity = perforModmoduleRepository.GetEntity(t => t.Id == request.ModuleId);
            if (entity == null)
                throw new PerformanceException("该绩效抽取方案不存在！");

            entity.ModuleName = request.ModuleName ?? entity.ModuleName;
            entity.SheetType = request.SheetType ?? entity.SheetType;
            entity.Description = request.Description ?? entity.Description;
            if (!perforModmoduleRepository.Update(entity))
                throw new PerformanceException("修改失败！");

            return entity;
        }

        /// <summary>
        /// 绩效模板删除
        /// </summary>
        /// <returns></returns>
        public void DelModule(int moduleId)
        {
            var entity = perforModmoduleRepository.GetEntity(t => t.Id == moduleId);
            if (entity == null)
                throw new PerformanceException("该绩效抽取方案不存在！");

            if (!perforModmoduleRepository.Remove(entity))
                throw new PerformanceException("删除失败！");
            if (entity.SheetType != (int)SheetType.SpecialUnit)
            {
                var itemList = perforModitemRepository.GetEntities(t => t.ModuleId == moduleId);
                if (itemList != null && itemList.Any())
                    perforModitemRepository.RemoveRange(itemList.ToArray());
            }
            else
            {
                var specialList = perforModspecialRepository.GetEntities(t => t.HospitalId == entity.HospitalId);
                if (specialList != null && specialList.Any())
                    perforModspecialRepository.RemoveRange(specialList.ToArray());
            }
        }

        /// <summary>
        /// 绩效收入模板配置项列表
        /// </summary>
        /// <returns></returns>
        public List<mod_item> Items(int moduleId)
        {
            var list = perforModitemRepository.GetEntities(t => t.ModuleId == moduleId);
            return list;
        }

        /// <summary>
        /// 绩效收入模板配置项新增
        /// </summary>
        /// <returns></returns>
        public void AddItem(ItemListRequest request)
        {
            var entity = perforModmoduleRepository.GetEntity(t => t.Id == request.ModuleId);
            if (entity == null)
                throw new PerformanceException("选择的绩效抽取方案不存在！");

            var list = request.Items;
            list.ForEach(t => { t.ModuleId = entity.Id; t.ExtractId = entity.ExtractId; });
            if (!perforModitemRepository.AddRange(list.ToArray()))
                throw new PerformanceException("添加失败！");
        }

        /// <summary>
        /// 绩效收入模板配置项修改
        /// </summary>
        /// <returns></returns>
        public mod_item EditItem(mod_item entity)
        {
            var item = perforModitemRepository.GetEntity(t => t.Id == entity.Id); if (item == null)
                throw new PerformanceException("选择修改的项不存在！");

            item.FactorValue1 = entity.FactorValue1;
            item.FactorValue2 = entity.FactorValue2;
            if (!perforModitemRepository.Update(item))
                throw new PerformanceException("修改失败！");

            return item;
        }

        /// <summary>
        /// 绩效收入模板配置项删除
        /// </summary>
        /// <returns></returns>
        public void DelItem(int itemId)
        {
            var item = perforModitemRepository.GetEntity(t => t.Id == itemId);
            if (item == null)
                throw new PerformanceException("Item项不存在！");

            perforModitemRepository.Remove(item);
        }

        /// <summary>
        /// 特殊科室模板配置项列表
        /// </summary>
        /// <returns></returns>
        public List<mod_special> Special(int hospitalId)
        {
            var list = perforModspecialRepository.GetEntities(t => t.HospitalId == hospitalId);
            return list;
        }

        /// <summary>
        /// 特殊科室模板配置项新增
        /// </summary>
        /// <returns></returns>
        public void AddSpecial(SpecialListRequest request)
        {

            var list = request.Items;
            if (!perforModspecialRepository.AddRange(list.ToArray()))
                throw new PerformanceException("添加失败！");
        }

        /// <summary>
        /// 特殊科室模板配置项修改
        /// </summary>
        /// <returns></returns>
        public mod_special EditSpecial(mod_special entity)
        {
            var special = perforModspecialRepository.GetEntity(t => t.Id == entity.Id);
            if (special == null)
                throw new PerformanceException("选择修改的数据不存在！");

            special.Department = entity.Department ?? special.Department;
            special.Target = entity.Target;
            special.TargetFactor = entity.TargetFactor;
            special.AdjustFactor = entity.AdjustFactor;
            if (!perforModspecialRepository.Update(special))
                throw new PerformanceException("修改失败！");

            return special;
        }

        /// <summary>
        /// 特殊科室模板配置项删除
        /// </summary>
        /// <returns></returns>
        public void DelSpecial(int specialId)
        {
            var special = perforModspecialRepository.GetEntity(t => t.Id == specialId);
            if (special == null)
                throw new PerformanceException("需要删除的项不存在！");

            perforModspecialRepository.Remove(special);
        }
    }
}
